#! /bin/sh
#
#   verify.sh -- データが消えているか確認する
#
#	使い方: /bin/sh verify.sh step ディスク
#		step: スキップする長さ
#		ディスク: デバイスファイルで指定、先頭の「/dev/」は省く
#		例: /bin/sh verify.sh 90 ad0
#
#	0.0: Jul.  8, 2005 by Dai ISHIJIMA (for wpout06)
#	0.2: Aug.  1, 2006 (タイムスタンプ for wpout07)
#	0.3: Feb. 10, 2007 (ログ記録 for wpout08)
#	0.4: Feb. 15, 2007 (エンターキー長押し対策)
#	0.5: Sep. 12, 2007 (wpout09)
#

step="${1:-1}"
disk="${2:-ad0}"
dmesg="${DMESG:-/var/run/dmesg.boot}"
# 10Mバイト単位で確認
vblklen="${VBLKLEN:-10485760}"
# 一度に1Mバイト
vbs="${VBS:-1048576}"
# 一度に10ブロック
vcnt=`expr $vblklen / $vbs`

if [ ! -r /dev/$disk ]; then
	echo "$0: disk '$disk' does not exist or is not readable"
	echo "Usage: $0 [step] disk"
	echo "Example: $0 -1 ad0"
	exit 1
fi

title="${TITLE:-ハードディスク消去ツール『wipe-out』}"

dname=`sed -n -E "/^${disk}:.*<.*>/p" $dmesg | sed 's/.*\(<.*>\).*/\1/'`
dbytes=`disksize -p /dev/$disk`


# 確認のパラメータ
bytes=`disksize -b /dev/$disk`
# ブロックサイズ単位でのディスクサイズ
max=`expr '(' $bytes + $vbs - 1 ')' / $vbs`

# 結果
zero=0
one=0
maybe=0
unerased=0
checked=0

startsec=`tinydate '+%s'`
rems=''

# 開始時刻
startstamp=`date '+%Y-%m-%d %H:%M'`

#
echo -n "verify started: $disk ($dname) $bytes bytes on " >> $LOGFIL
date >> $LOGFIL

nerrs=0
skip=0
while [ $skip -lt $max ]; do
	# 計時
	cursec=`tinydate '+%s'`
	# バーグラフ
	p=`expr 100 '*' $skip / $max`
	m=`expr 1000 '*' $skip / $max`
	q=`expr $p / 2`
	i=0
	bar=''
	while [ $i -lt $q ]; do
		bar="$bar"'#'
		i=`expr $i + 1`
	done
	while [ $i -lt 50 ]; do
		bar="$bar"'_'
		i=`expr $i + 1`
	done
	# 残り時間
	elapsed=`expr $cursec - $startsec`
	if [ $elapsed -gt 10 ]; then
		remain=`expr '(' 1000 - $m ')' '*' $elapsed / $m + 1`
		if [ $remain -gt 120 ]; then
			rems=`expr '(' $remain + 59 ')' / 60`
			rems="残り およそ${rems}分"
		else
			rems="残り およそ${remain}秒"
		fi
	fi
	if [ $elapsed -gt 120 ]; then
		elas=`expr '(' $elapsed + 30 ')' / 60`
		elas="${elas}分"
	else
		elas="${elapsed}秒"
	fi
	#
	text="ディスク ${disk} のデータをチェックしています。\n"
	text="${text}${max}ブロックのうち${skip}ブロック／${p}% 終了\n"
	text="${text}${elas} 経過  ${rems}  \n\n"
	text="${text}        0%|${bar}|100%"
	text="${text}\n${errormsg}"
	dialog --title "$title" --infobox "$text" 8 72
	echo -n "$status2"
	case x$DEBUG in
		x[Yy]*)
			status2='now in debug mode...'
			errormsg=''
			sleep 1
			;;
		*)
			status=`(dd if=/dev/$disk ibs=$vbs skip=$skip count=$vcnt | check -s) 2>&1`
			case $? in
				0) zero=`expr $zero + $vcnt` ;;
				1) one=`expr $one + $vcnt` ;;
				2) maybe=`expr $maybe + $vcnt` ;;
				*) unerased=`expr $unerased + $vcnt` ;;
			esac
			status2=`echo "$status" | sed -n '/transfer/p'`
			errormsg=`echo "$status" | sed -n -E '/records (in|out)$/!p' | sed -n '/transferred in/!p'`
			;;
	esac
	# 記録
	case x"$skip" in
		x0)
			echo "skip: $skip, $status2" >> $LOGFIL
			;;
	esac
	case x"${nerrs}"x"${errormsg}" in
		x[0-8]x?*)
			echo "$errormsg" >> $LOGFIL
			nerrs=`expr $nerrs + 1`
			;;
		x9x?*)
			echo "$errormsg" >> $LOGFIL
			echo "Too many errors encounterd..." >> $LOGFIL
			nerrs=`expr $nerrs + 1`
			;;
	esac
	# 次ループ
	checked=`expr $checked + $vcnt`
	skip=`expr $skip + $vcnt + $step`
done

sleep 2

#
echo "skip: $skip, $status2" >> $LOGFIL
echo -n "verify finished: $disk ($dname) $bytes bytes on " >> $LOGFIL
date >> $LOGFIL

cursec=`tinydate '+%s'`
elapsed=`expr $cursec - $startsec`
if [ $elapsed -gt 120 ]; then
	min=`expr $elapsed / 60`
	sec=`expr $elapsed - $min '*' 60`
	elas="${min}分${sec}秒"
else
	elas="${elapsed}秒"
fi

# 終了時刻
endstamp=`date '+%H:%M'`

unerased=`expr $unerased + $maybe`
zp=`expr $zero '*' 100 / $checked`
op=`expr $one '*' 100 / $checked`
np=`expr $unerased '*' 100 / $checked`

msg="ディスク ($disk, $dbytes バイト) のチェックが終了しました (${elas})\n"
msg="${msg}全 $max ブロック中の $checked ブロックを検査しました\n"
msg="${msg}  '0' で消去されているブロック数: $zero ($zp %)\n"
msg="${msg}  '1' で消去されているブロック数: $one ($op %)\n"
msg="${msg}  それ以外 (未消去) のブロック数: $unerased ($np %)\n"

# ログ保存
echo "Verified $checked blocks" > /tmp/verify-log.$disk
echo -n " erased with 0:${zp}%, 1:${op}%, " >> /tmp/verify-log.$disk
echo "unerased:${np}%" >> /tmp/verify-log.$disk
echo " $startstamp -> $endstamp" >> /tmp/verify-log.$disk

sh dummyread.sh
dialog	--title "$title" \
	--msgbox "$msg" 10 72

exit 0

# EOF
