#! /bin/sh
#
#   startup.sh -- スタートアップスクリプト (日本語化コンソール用)
#
#	0.0: Jun. 15, 2004 by Dai ISHIJIMA
#	0.1: Jun. 19, 2005 (for PXEBOOT)
#	0.2: Aug.  1, 2006 (v07、CD初期化をrcへ、ランダムパターンとその補数)
#	0.3: Aug.  6, 2006 (キーボードチェック)
#	0.4: Sep. 18, 2006 (/var を mkdir しない)
#	0.5: Feb. 10, 2007 (ログ記録 for wpout08)
#	0.6: Sep. 12, 2007 (for wpout09開発版)
#

##PATH=/usr/bin:/bin:/usr/sbin:/sbin:/:.
##export PATH

DMESG=/var/run/dmesg.boot
export DMESG
echo -n '起動時のシステムメッセージを保存しています...'
##mkdir -p /var/run
##mkdir -p /var/tmp
dmesg > $DMESG
echo ''

echo -n "script startup.sh started on " >> $LOGFIL
date >> $LOGFIL

# キーボードのチェック
kbd=`cat /var/run/dmesg.boot | sed -n '/^kbd[1-9] at/p' | sed 's/\(kbd[1-9]\) at .*/\1/' | sed -n '$p'`

KEYDEV=/dev/kbd0
##case x"$kbd" in
##    xkbd[1-9])
##	/usr.sbin/kbdcontrol -k /dev/$kbd < /dev/console
##	keydev=`cat /var/run/dmesg.boot | sed "/^${kbd}: /p" | sed -n '$p'`
##	echo "キーボードを '$keydev' に変更しました"
##	KEYDEV=/dev/$kbd
##	echo "keyboard device was changed to $KEYDEV" >> $LOGFIL
##	;;
##esac
export KEYDEV

echo -n 'ハードディスク消去ツールの起動準備をしています...'
##if [ -r /usr/lib/libc.so ]; then
##	# シェアードライブラリがあれば (CDのマウントに成功していれば) 
##	# ライブラリの初期化
##	ldconfig -elf /usr/lib
##fi

##if [ -x /bin/hostname ]; then
##	hostname wipeout
##fi

# 一度CD-ROMをアンマウント
##case $CDROM in
##	/dev/*)
##		echo "CD-ROM ($CDROM) をアンマウントしています..."
##		umount /cdrom
##		;;
##esac

export TERMCAP=/etc/termcap
export TERM=kons25jx
export PATTERNFIL=/tmp/pattern.dat
export DEVICES='da|ad|ar|aacd|amrd'

# 一度に32Mバイトずつ消す
BLKLEN=$(( 32 '*' 1024 '*' 1024 ))
export BLKLEN
# 出力ブロック長は1Mバイト
OBS=$(( 1024 '*' 1024 ))
export OBS
# 入力ブロック長 (stuffの出力長) は1kバイト
IBS=1024
export IBS
# 消去確認は10Mバイトずつ
VBLKLEN=$(( 10 '*' 1024 '*' 1024 ))
export VBLKLEN
# 消去確認の入力ブロック長は1Mバイト
VBS=$(( 1024 '*' 1024 ))
export VBS
# コピー時は512バイトずつ
CBS=512
export CBS

echo ''

# debugging
case x"$DEBUG" in
    x[Yy]*)
	for i in 1 2 3 ; do
		echo -n '.'
		sleep 1
	done
	echo ''
	;;
esac

cd /wipeout
stuff -g > $PATTERNFIL
/bin/sh mainmenu.sh

# とりあえず対話シェルを動かしておく :-)
/bin/sh -i

exit 0

# EOF
